/**
 * @file	cf_socket.h
 * @author	myusgun <myusgun@gmail.com>
 * @version	0.1
 *
 * @remark TCP 소켓만 지원됨
 *
 * @example socket_server.c
 * @example socket_client.c
 */
#ifndef __CF_SOCKET_H__
#define __CF_SOCKET_H__

#include "cf_base.h"

#include <stddef.h>

/** 타임아웃을 설정하지 않음 */
#define CF_SOCKET_NO_TIMEOUT		-1

/** 호스트 정보 */
typedef struct cf_socket_hostinfo {
	char			address[128];	/**< 원격 호스트 주소 */
	unsigned short	port;			/**< 원격 호스트 포트 */
} CF_Socket_HostInfo;

#ifdef __cplusplus
extern "C" {
#endif

CF_EXPORT CF_BOOL
CF_Socket_IsInitialized		(void);

CF_EXPORT int
CF_Socket_Initialize		(void);

CF_EXPORT int
CF_Socket_Finalize			(void);

CF_EXPORT int
CF_Socket_Close				(const int sock);

CF_EXPORT int
CF_Socket_SetOption			(const int		sock,
							 const int		optname,
							 const void		* optval,
							 const size_t	optlen);

CF_EXPORT int
CF_Socket_GetOption			(const int	sock,
							 const int	optname,
							 void		* optval,
							 size_t		* optlen);

CF_EXPORT int
CF_Socket_Connect			(const char				* ip,
							 const unsigned short	port);

CF_EXPORT int
CF_Socket_ConnectTimeout	(const char				* ip,
							 const unsigned short	port,
							 const int				timeout);

CF_EXPORT int
CF_Socket_Server			(const unsigned short	port,
							 const int				backlog);

CF_EXPORT int
CF_Socket_Accept			(const int			sock,
							 CF_Socket_HostInfo	* host);

CF_EXPORT int
CF_Socket_Send				(const int	sock,
							 const void	* buf,
							 const int	len);

CF_EXPORT int
CF_Socket_SendTimeout		(const int	sock,
							 const void	* buf,
							 const int	len,
							 const int	timeout);

CF_EXPORT int
CF_Socket_Recv				(const int	sock,
							 void		* buf,
							 const int	len);

CF_EXPORT int
CF_Socket_RecvTimeout		(const int	sock,
							 void		* buf,
							 const int	len,
							 const int	timeout);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __CF_SOCKET_H__
